VERSION 5.00
Begin VB.UserControl myTextBox 
   ClientHeight    =   345
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   2220
   ScaleHeight     =   345
   ScaleWidth      =   2220
   Begin VB.TextBox Text1 
      Alignment       =   1  'Right Justify
      Height          =   315
      Left            =   0
      TabIndex        =   0
      Text            =   "Text1"
      Top             =   0
      Width           =   2145
   End
End
Attribute VB_Name = "myTextBox"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ---------------------------------------------------------------------------------
'    Disclaimer:  The code is provided "as-is" by the Author,
'    who is not responsible for any harm to computer systems or software
'    with which it is used. You are free to modify the code and/or use it
'    yourself in any way you see fit.
'    You are free to redistribute the package under the following conditions:
'       1) appropriate credit is given to author,
'       2) the code is provided unaltered and in full, and
'       3) you do not charge any kind of fee for the code or its binary version
'          without the express permission of the author.
'
'    If you change this code please, please, p-l-e-a-s-e send me
'    a copy. I have just strated using classes and any help will be appriciated.
'
'    Please email your comments to paulo.batista@lineone.net
'
' ---------------------------------------------------------------------------------
'
'
' Author : Paulo Batista
'
' Date   : 16/09/2000

' Purpose
' ========
' This user defined control adds some functionality to the standard TextBox Control
'
' Added Properties
' ================
' 1. OldValue -
'    A private property that stores the value when the myTextbox gets focus
'    The old value may be rescued by pressing the Esc key.
'
' 2. FocusBackColor / FocusForeColor
'    Public properties that change the BackColor and Forecolor of the myTextBox when
'    it gets the focus. Focus Colours default to and BackColor and Forecolor so you
'    need to change it if you want to see a difference in behaviour
'
' 3. DataType ' Main reason to create this control
'    Public property that can be set to a range of values namely [Text, Integer, Positive,
'     Negative, Single, Double, Currency, Decimal, and Byte]
'    It automatically checks for valid input range
'
' 4. Min/Max property
'    Public property that determines a valid range of numbers
'    If set to anything either then 0 it will bypass Datatype check and validate then entry
'    based on min and max values. Not applicable to text datatype
'
' 5. AssociatedLabel
'    Public property that when set to something other then an empty string, it will loop through
'    all controls trying to find the label that maches the string in AssociatedLabel
'    If it finds one it changes its fontbold property to true
'    NB. it does not work (not yet anyway) if labels are index.

'    Idealy I would like to get all labels on the form and add then to the AssociatedLabel list
'    if anyone knows how to do it please let me know by email
'                       paulo.batista@lineone.net
'
' ----------------------------------------------------------------------------------


Option Explicit

Private iValue

Private oValue As String
Private fFocusBackColor As OLE_COLOR
Private fBackColor As OLE_COLOR
Private fFocusForeColor As OLE_COLOR
Private fForeColor As OLE_COLOR

Private minVal As Long
Private maxVal As Long

Private aLabel As String

Private mDataType As DataTypes
Public Enum DataTypes
    [tText] = 0
    [tInteger] = 1
    [tLong] = 2
    [tPositive] = 3
    [tNegative] = 4
    [tCurrency] = 5
    [tSingle] = 6
    [tDouble] = 7
    [tDecimal] = 8
    [tByte] = 9
End Enum

'EVENTS
Public Event Change()
Public Event Click()
Public Event DblClick()
Public Event KeyDown(KeyCode As Integer, Shift As Integer)
Public Event KeyUp(KeyCode As Integer, Shift As Integer)
Public Event KeyPress(KeyAscii As Integer)
Public Event MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
Public Event MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
Public Event MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)

Private Sub UserControl_Initialize()
    Text1.ZOrder 0
    Text1.Text = ""
    fFocusBackColor = Text1.BackColor
    fFocusForeColor = Text1.ForeColor
    fBackColor = Text1.BackColor
    fForeColor = Text1.ForeColor
    aLabel = ""
    Text1.Alignment = 0
End Sub

Private Sub Text1_GotFocus()
    Text1.SelStart = 0
    Text1.SelLength = Len(Text1)
    Text1.BackColor = FocusBackColor
    Text1.ForeColor = FocusForeColor
    oValue = Text1.Text
    AssociatedLabelFontBold True
End Sub

Private Sub Text1_KeyPress(KeyAscii As Integer)
    
    If KeyAscii = 27 Then ' ESC key
        ' restore old value
        Text1.Text = oValue
        Exit Sub
    End If
    '
    Select Case mDataType
        ' ignores case [0 - Text] but validates any other data type
        Case 1 To 9: If Not IsValidCharacter(KeyAscii) Then KeyAscii = 0
    End Select
    
End Sub

Private Sub Text1_LostFocus()
Dim isValidEntry As Boolean
Dim i As Boolean
Dim Msg As String
    '
    ' Validates input
    ' If Min/Max is set to 0(default) the validation is performed acording to the datatype
    ' otherwise validation is based on Min/Max values
    '
    On Error GoTo Text1_LostFocus
    ' ignore empty stings
    If Trim(Text1.Text) <> "" Then
        isValidEntry = True ' be positive
        ' is min/max set to anything other then 0 ?
        If (MinValue = 0 And MaxValue = 0) And (mDataType <> 0) Then
            ' check input based on current data type
            ' note. cPos and cNeg are user defined function that check
            ' for positive and negative and raises an error if required
            Select Case mDataType
                Case 1: i = CInt(Text1.Text)
                Case 2: i = CLng(Text1.Text)
                Case 3: i = cPos(Text1.Text)
                Case 4: i = cNeg(Text1.Text)
                Case 5: i = CCur(Text1.Text)
                Case 6: i = CSng(Text1.Text)
                Case 7: i = CDbl(Text1.Text)
                Case 8: i = CDec(Text1.Text)
                Case 9: i = CByte(Text1.Text)
            End Select
        Else
            ' check min/max range
            Msg = ""
            If mDataType <> 0 Then
                If (CLng(Text1.Text) < minVal) Then
                    Msg = "Invalid entry. Minimum allowed is " & minVal & " !"
                ElseIf (CLng(Text1.Text) > maxVal) Then
                    Msg = "Invalid entry. Max allowed is " & maxVal & " !"
                End If
                If Msg <> "" Then
                    MsgBox Msg, vbExclamation
                    Text1.SetFocus
                    GoTo Text1_LostFocus_Exit
                End If
            End If
        End If
    End If
    ' reset fore and backcolor
    Text1.BackColor = fBackColor
    Text1.ForeColor = fForeColor
    AssociatedLabelFontBold False
Text1_LostFocus_Exit:
    Exit Sub
    
Text1_LostFocus:
    If Err.Number <> 0 Then
        If Err.Number = 6 Or Err.Number = 13 Then
            MsgBox "Invalid entry! " & Str(mDataType), vbExclamation
            Text1.SetFocus
            Resume Text1_LostFocus_Exit
        Else
            MsgBox Err.Number & ":" & Err.Description
            Text1.SetFocus
            Resume Text1_LostFocus_Exit
        End If
    End If
End Sub

Private Sub AssociatedLabelFontBold(NewValue As Boolean)
' searches for a label name that matches the one in AssociatedLabel property
' if found sets its fontbold property to NewValue(True/False)
Dim i As Long
    For i = 0 To Parent.Controls.Count - 1
        If LCase(Parent.Controls(i).Name) = LCase(aLabel) Then
            Parent.Controls(i).FontBold = NewValue
            Exit Sub
        End If
    Next
End Sub
' resise to fit
Private Sub UserControl_Resize()
    Text1.Move 0, 0, Width, Height
End Sub

' Define PROPERTIES for custom control
Public Property Get Enabled() As Boolean
Attribute Enabled.VB_ProcData.VB_Invoke_Property = ";Behavior"
Attribute Enabled.VB_UserMemId = -514
    Enabled = UserControl.Enabled
End Property

Public Property Let Enabled(ByVal NewValue As Boolean)
    UserControl.Enabled = NewValue
    PropertyChanged "Enabled"
    Text1.Enabled = NewValue
End Property

Public Property Get Text() As String
    Text = Text1.Text
End Property

Public Property Let Text(ByVal NewValue As String)
    PropertyChanged "Text"
    Text1.Text = NewValue
End Property

Public Property Set Font(ByVal NewValue As StdFont)
    Set Text1.Font = NewValue
    PropertyChanged "Font"
End Property

Public Property Get Font() As StdFont
    Set Font = Text1.Font
End Property

Public Property Get FontName() As String
    FontName = Text1.FontName
End Property

Public Property Let FontName(ByVal NewValue As String)
    Text1.FontName = NewValue
    PropertyChanged "FontName"
End Property

Public Property Get FontBold() As Boolean
    FontBold = Text1.FontBold
End Property

Public Property Let FontBold(ByVal NewValue As Boolean)
    Text1.FontBold = NewValue
    PropertyChanged "FontBold"
End Property

Public Property Get FontItalic() As Boolean
    FontItalic = Text1.FontItalic
End Property

Public Property Let FontItalic(ByVal NewValue As Boolean)
    Text1.FontItalic = NewValue
    PropertyChanged "FontItalic"
End Property

Public Property Get FontUnderline() As Boolean
    FontUnderline = Text1.FontUnderline
End Property

Public Property Get FontStrikethru() As Boolean
    FontStrikethru = Text1.FontStrikethru
End Property

Public Property Let FontStrikethru(ByVal NewValue As Boolean)
    Text1.FontStrikethru = NewValue
    PropertyChanged "FontStrikethru"
End Property

Public Property Let FontSize(NewValue As Single)
    Text1.FontSize = NewValue
    PropertyChanged "FontSize"
End Property

Public Property Let FontUnderline(ByVal NewValue As Boolean)
    Text1.FontUnderline = NewValue
    PropertyChanged "FontUnderline"
End Property

Public Property Get ForeColor() As OLE_COLOR
    ForeColor = Text1.ForeColor
End Property

Public Property Let ForeColor(ByVal NewValue As OLE_COLOR)
    Text1.ForeColor = NewValue
    PropertyChanged "ForeColor"
End Property

Public Property Get FontSize() As Single
    FontSize = Text1.FontSize
End Property

Public Property Get MaxLength() As Long
    MaxLength = Text1.MaxLength
End Property

Public Property Let MaxLength(ByVal NewValue As Long)
    Text1.MaxLength = NewValue
    PropertyChanged "MaxLength"
End Property

Public Property Get SelStart() As Long
    SelStart = Text1.SelStart
End Property

Public Property Let SelStart(ByVal NewValue As Long)
    Text1.SelStart = NewValue
End Property

Public Property Get SelLength() As Long
    SelLength = Text1.SelLength
End Property

Public Property Let SelLength(ByVal NewValue As Long)
    Text1.SelLength = NewValue
End Property

Public Property Get SelText() As String
    SelText = Text1.SelText
End Property

Public Property Let SelText(ByVal NewValue As String)
    Text1.SelText = NewValue
End Property

Public Property Get BackColor() As OLE_COLOR
    BackColor = Text1.BackColor
End Property

Public Property Let BackColor(ByVal NewValue As OLE_COLOR)
    Text1.BackColor = NewValue
    PropertyChanged "BackColor"
End Property

Public Property Get FocusBackColor() As OLE_COLOR
    FocusBackColor = fFocusBackColor
End Property

Public Property Let FocusBackColor(ByVal NewValue As OLE_COLOR)
    fFocusBackColor = NewValue
    PropertyChanged "FocusBackColor"
End Property

Public Property Get FocusForeColor() As OLE_COLOR
    FocusForeColor = fFocusForeColor
End Property

Public Property Let FocusForeColor(ByVal NewValue As OLE_COLOR)
    fFocusForeColor = NewValue
    PropertyChanged "FocusForeColor"
End Property

Public Property Get AssociatedLabel() As String
    AssociatedLabel = aLabel
End Property

Public Property Let AssociatedLabel(ByVal NewValue As String)
    aLabel = NewValue
    PropertyChanged "AssociatedLabel"
End Property

Private Property Get OldValue() As String
    oValue = Text1.Text
End Property

Private Property Let OldValue(ByVal NewValue As String)
    oValue = NewValue
    PropertyChanged "OldValue"
End Property

Public Property Get Alignment() As AlignmentConstants
    Alignment = Text1.Alignment
End Property

Public Property Let Alignment(ByVal NewValue As AlignmentConstants)
    Text1.Alignment = NewValue
    PropertyChanged "Alignment"
End Property

Public Property Get MultiLine() As Boolean
    MultiLine = Text1.MultiLine
End Property

Public Property Let MultiLine(ByVal NewValue As Boolean)
    'Text1.MultiLine = NewValue
    PropertyChanged "MultiLine"
End Property

Public Property Get MinValue() As Long
    MinValue = minVal
End Property

Public Property Let MinValue(ByVal NewValue As Long)
    ' checks that the NewValue is Less or equal to MaxValue
    If DataType <> 0 Then
        If NewValue > MaxValue And maxVal <> 0 Then
            MsgBox "Invalid Property Value. Min > Max", vbExclamation
        Else
            minVal = NewValue
        End If
        PropertyChanged "MinValue"
    End If
End Property

Public Property Get MaxValue() As Long
    MaxValue = maxVal
End Property

Public Property Let MaxValue(ByVal NewValue As Long)
    ' checks that the NewValue is greater or equal to MinValue
    ' ignores Min/Max if datatype is set to Text
    If DataType <> 0 Then
        If NewValue < MinValue And MinValue <> 0 Then
            MsgBox "Invalid Property Value. Max < Min", vbExclamation
        Else
            maxVal = NewValue
        End If
        
        PropertyChanged "MaxValue"
    End If
End Property

Public Property Get DataType() As DataTypes
    DataType = mDataType
End Property

Public Property Let DataType(ByVal vNewValue As DataTypes)
    ' if datatype is Text sets the Allignment and resets the Min/Max values
    ' ignores Min/Max if datatype is set to Text
    mDataType = vNewValue
    UserControl.PropertyChanged "DataType"
    If mDataType <> 0 Then
        Text1.Alignment = 1 ' Right justify
    Else
        Text1.Alignment = 0 ' Left justify for text only
        MinValue = 0
        MaxValue = 0
    End If
End Property

' Read and Write Properties
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    With PropBag
        Alignment = .ReadProperty("Alignment", Text1.Alignment)
        AssociatedLabel = .ReadProperty("AssociatedLabel", aLabel)
        BackColor = .ReadProperty("BackColor", Text1.BackColor)
        DataType = .ReadProperty("DataType", 0)
        Enabled = .ReadProperty("Enabled", True)
        FocusBackColor = .ReadProperty("FocusBackColor", Text1.BackColor)
        FocusForeColor = .ReadProperty("FocusForeColor", Text1.ForeColor)
        FontBold = .ReadProperty("FontBold", False)
        FontItalic = .ReadProperty("FontItalic", False)
        FontName = .ReadProperty("FontName", "Ms Sans Serif")
        FontSize = .ReadProperty("FontSize", 8)
        FontStrikethru = .ReadProperty("FontStrikethru", False)
        FontUnderline = .ReadProperty("FontUnderline", False)
        ForeColor = .ReadProperty("ForeColor", Text1.ForeColor)
        MaxLength = .ReadProperty("MaxLength", 0)
        
        MinValue = .ReadProperty("MinValue", 0)
        MaxValue = .ReadProperty("MaxValue", 0)
        
        MultiLine = .ReadProperty("MultiLine", Text1.MultiLine)
        OldValue = .ReadProperty("AssociatedLabel", oValue)
        Text = .ReadProperty("Text", Text1.Text)
    End With
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    With PropBag
        .WriteProperty "Alignment", Alignment, 0
        .WriteProperty "AssociatedLabel", aLabel, ""
        .WriteProperty "BackColor", BackColor
        .WriteProperty "DataType", DataType, 0
        .WriteProperty "Enabled", Enabled, True
        .WriteProperty "FocusBackColor", fFocusBackColor
        .WriteProperty "FocusForeColor", fFocusForeColor
        .WriteProperty "FontBold", FontBold, False
        .WriteProperty "FontItalic", FontItalic
        .WriteProperty "FontName", FontName, "Ms Sans Serif"
        .WriteProperty "FontSize", FontSize, 8
        .WriteProperty "FontStrikethru", FontStrikethru, False
        .WriteProperty "FontUnderline", FontUnderline, False
        .WriteProperty "ForeColor", ForeColor
        .WriteProperty "MaxLength", MaxLength, 0
        
        .WriteProperty "MinValue", minVal, 0
        .WriteProperty "MaxValue", maxVal, 0
        
        .WriteProperty "MultiLine", MultiLine, False
        .WriteProperty "Text", Text, 0
    End With
End Sub

' Add Events Handlers

Private Sub Text1_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    RaiseEvent MouseDown(Button, Shift, X, Y)
End Sub

Private Sub Text1_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    RaiseEvent MouseMove(Button, Shift, X, Y)
End Sub

Private Sub Text1_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    RaiseEvent MouseUp(Button, Shift, X, Y)
End Sub

Private Sub Text1_KeyUp(KeyCode As Integer, Shift As Integer)
    RaiseEvent KeyUp(KeyCode, Shift)
End Sub
Private Sub Text1_KeyDown(KeyCode As Integer, Shift As Integer)
    RaiseEvent KeyDown(KeyCode, Shift)
End Sub
Private Sub Text1_Change()
    RaiseEvent Change
End Sub
Private Sub Text1_DblClick()
    RaiseEvent DblClick
End Sub
Private Sub Text1_Click()
    RaiseEvent Click
End Sub


' Private Function
Private Function IsValidCharacter(KeyAscii As Integer) As Boolean
' check key pressed. returns false if not in range of numbers$
    Const Numbers$ = "0123456789-."
    IsValidCharacter = True
    If KeyAscii <> 8 Then
        If (InStr(Numbers, Chr(KeyAscii)) = 0) Then
            IsValidCharacter = False
            Exit Function
        End If
    End If
End Function

Private Function cPos(txt) As Boolean
    ' checks that the value of txt is positive. if not raised an error
    If Not CLng(txt) >= 0 Then Err.Raise 6
End Function

Private Function cNeg(txt) As Boolean
    ' checks that the value of txt is negative. if not raised an error
    If Not CLng(txt) <= 0 Then Err.Raise 6
End Function

